/*
Presto - An Allegro tilemap library
Copyright (C) 2008  Peter Fernandes

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

functions.c: functions to set up or change a tilemap
*/


#include "presto.h"

//changes the animation that the tile follows.  -1 to stop animation
//also resets the tile index to 0 for the start of the new animation and gives it the default loop value for that animation
void tile_change_anim(TILEMAP * tilemap, unsigned int x, unsigned int y, int id) {
	int _index=(y*tilemap->width)+x;
	
	tilemap->map[_index].orderid=id;
	tilemap->map[_index].index=0;
	tilemap->map[_index].wait=0;
	
	if(id>=0) {
		tilemap->map[_index].tileid=tilemap->orders[id][0];
		tilemap->map[_index].loop=tilemap->loopvals[id];
	}
	
}

//changes the animation step that the tile is on if animated, or changes the current sprite if static.
//In the latter case, index refers to the id of the bitmap in the pool
//NOTE: If the tile is animated and synced to the frameclock, this function will have no effect
void tile_change_index(TILEMAP * tilemap, unsigned int x, unsigned int y, unsigned int index) {
	int _index=(y*tilemap->width)+x;
	
	if(tilemap->map[_index].orderid>=0) {
		tilemap->map[_index].index=index;
		tilemap->map[_index].wait=0;
		tilemap->map[_index].tileid=tilemap->orders[tilemap->map[_index].orderid][index];
	}
	else {
		tilemap->map[_index].tileid=index;
	}
	
	
	
}


//changes the loop parameter for an animated tile.
//Set loop for -1 for infinite loop, 0 to stop looping, or X where X>0 to loop X times.
void tile_set_loop(TILEMAP * tilemap, unsigned int x, unsigned int y, int loop) {
	int _index=(y*tilemap->width)+x;
	
	tilemap->map[_index].loop=loop;
	
	
}




//sets whether the tile should sync with the frameclock.  0==No, !0=yes
void tile_set_sync(TILEMAP * tilemap, unsigned int x, unsigned int y, int sync) {
	int _index=(y*tilemap->width)+x;
	
	tilemap->map[_index].sync_to_frameclock=sync;
	
}


//sets the transparency level of the tile in the pool.  255==opaque, 0==transparent
//-2==ignore the alpha channel completely (no transparency)
//-1==read alpha values from the file (but this has no effect after the bitmap is loaded)
void tile_set_trans(TILEMAP * tilemap, unsigned int index, int trans) {
	int x;
	int y;
	int newtrans=trans;

	drawing_mode(DRAW_MODE_TRANS,NULL,0,0);
	set_write_alpha_blender();
	
	tilemap->trans[index]=trans;
	
	if(trans==-2)
		newtrans=255;
	
	for (y=0;y<tilemap->pool[index]->h;y++) {
		for (x=0;x<tilemap->pool[index]->w;x++) {
			if(geta(getpixel(tilemap->pool[index],x,y)))
				putpixel(tilemap->pool[index],x,y,tilemap->trans[index]);
		}
	}

}


//changes an animation sequence currently stored in the tilemap or adds a new one if index <0
//Returns the index of the animation in the animation list or <0 on failure
int tilemap_change_anim(TILEMAP *tilemap, int index, unsigned int *order, unsigned int *delays, int loop, unsigned int count) {
	int _index;
	
	
	if(index<0) {
		if(!(tilemap->orders=realloc(tilemap->orders,(tilemap->num_anims+1)*sizeof(unsigned int *))) ||
		!(tilemap->delays=realloc(tilemap->delays,(tilemap->num_anims+1)*sizeof(unsigned int *))) ||
		!(tilemap->loopvals=realloc(tilemap->loopvals,(tilemap->num_anims+1)*sizeof(unsigned int))) ||
		!(tilemap->num_frames=realloc(tilemap->num_frames,(tilemap->num_anims+1)*sizeof(unsigned int)))) {
			return -2;
		}
		
		_index=tilemap->num_anims;
		
		tilemap->num_anims++;
	}
	else {
		_index=index;
	}

	memcpy(tilemap->orders[_index],order,count*sizeof(unsigned int));
	memcpy(tilemap->delays[_index],delays,count*sizeof(unsigned int));
	tilemap->loopvals[_index]=loop;
	tilemap->num_frames[_index]=count;
	
	
	return _index;
}


//changes a bitmap at index in the pool, or adds a new one if index <0
//Returns the index of the bitmap or -2 on memory failure, -4 if error loading the bitmap
int tilemap_change_bitmap(TILEMAP *tilemap, int index, char *filename) {
	int _index;
	
	if(index<0) {
		
		if(!(tilemap->pool=realloc(tilemap->pool,(tilemap->poolsize+1)*sizeof(BITMAP *)))) {
			return -2;
		}
		
		_index=tilemap->poolsize;
		
		tilemap->poolsize++;
	}
	else {
		_index=index;
	}
	
	if(!(tilemap->pool[index]=load_bitmap(filename,NULL))) {
		//try to size the pool back down
		tilemap->pool=realloc(tilemap->pool,(tilemap->poolsize-1)*sizeof(BITMAP *));
		return -4;
	}
	
	return _index;
}

//changes the dimensions of a tilemap
//Returns 0 on success, -1 on failure
//NOTE: This will invalidate the entire tilemap of TILE entries, so it is advised to only call this
//      function ONCE when first setting up the tilemap BEFORE any tile initializations
int tilemap_create_dims(TILEMAP *tilemap, unsigned int h, unsigned int w) {
	
	if(!(tilemap->map=malloc(sizeof(TILE)*h*w))) {
		return -2;
	}
	
	tilemap->height=h;
	tilemap->width=w;
	
	return 0;
}


		
